@@ -29,7 +29,6 @@
 #include <glade/glade.h>
 
 extern General 	 *general_setting;
-gint bm_pid;
 static gboolean	 general_changed = FALSE;
 /**
  * CheckButtons
@@ -103,68 +102,6 @@ genui_apply_sensitivity (gboolean state)
 {
     general_changed = state;
 }
-/**
- *
- * Launch in executions BrlMonitor
- *
-**/
-gboolean 
-genui_run_brlmonitor (void)
-{
-    /*
-	Load IP from gconf and test if it is not a localhost ip
-    */
-    gchar* ip = srcore_ip_get ();
-    if (strcmp (ip, DEFAULT_BRLMON_IP)) 
-	return FALSE;
-
-
-    if (g_file_test ("../brlmon/brlmonitor", G_FILE_TEST_EXISTS) &&
-        g_file_test ("../brlmon/brlmonitor", G_FILE_TEST_IS_EXECUTABLE) &&
-        g_file_test ("../brlmon/brlmonitor", G_FILE_TEST_IS_REGULAR))
-    {
-	gchar *args[] = {"../brlmon/brlmonitor", NULL};
-	if (!g_spawn_async ( ".", args , NULL , 
-			    G_SPAWN_DO_NOT_REAP_CHILD,
-			    NULL, NULL, &bm_pid, NULL))
-	{
-	    sru_message ("No \"%s\" binary file found [errno: %i].", 
-			    "brlmonitor",
-			     errno);
-	    return FALSE;
-	}
-    }
-    else
-    if (g_file_test ("./brlmonitor", G_FILE_TEST_EXISTS) &&
-        g_file_test ("./brlmonitor", G_FILE_TEST_IS_EXECUTABLE) &&
-        g_file_test ("./brlmonitor", G_FILE_TEST_IS_REGULAR))
-    {
-	gchar *args[] = {"./brlmonitor", NULL};
-	if (!g_spawn_async ( ".", args , NULL , 
-			    G_SPAWN_DO_NOT_REAP_CHILD,
-			    NULL, NULL, &bm_pid, NULL))
-	{
-	    sru_message ("No \"%s\" binary file found [errno: %i].", 
-			    "brlmonitor",
-			    errno);
-	    return FALSE;
-	}	
-    }
-    else
-    {
-	gchar *args[] = {"brlmonitor", NULL};
-	if (!g_spawn_async ( ".", args , NULL , 
-			    G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
-			    NULL, NULL, &bm_pid, NULL))
-	{
-	    sru_message ("No \"%s\" binary file found [errno: :%i]",
-			    "brlmonitor",
-			    errno);
-	    return FALSE;
-	}
-    }
-    return TRUE;
-}
 
 void 
 genui_changeing (void)
@@ -195,17 +132,6 @@ genui_changeing (void)
     {
 	general_setting->braille_monitor = check_val;
 	srcore_braille_monitor_status_set (general_setting->braille_monitor);
-	if (general_setting->braille_monitor) 
-	{
-	    genui_run_brlmonitor ();
-	    sleep (1);
-	    gtk_window_present ( GTK_WINDOW (w_general_settings));
-	}
-	else 
-	{
-	    kill (bm_pid, SIGTERM);
-	    bm_pid = 0;
-	}
     }
     
     check_val = 
