@@ -135,7 +135,7 @@ magui_cursor_off (gboolean val)
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_cursor_color), val);
     
     if (val)
-	cursor_size = !(magnifier_setting->cursor_scale);
+	cursor_size = magnifier_setting->cursor_scale;
 		
     gtk_widget_set_sensitive (GTK_WIDGET (sp_cursor_size), cursor_size);
     gtk_widget_set_sensitive (GTK_WIDGET (lb_cursor_size), cursor_size);
@@ -170,6 +170,13 @@ magui_crosswire_off (gboolean val)
     gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire_clip), val);
 }
 
+static void
+magui_crosswire_color_invert_off (gboolean val)
+{
+    gtk_widget_set_sensitive ( GTK_WIDGET (cp_crosswire_color), val);
+    gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_color), val);   
+}
+
 #if 0
 void 
 magui_save_magnifier_setting_value (Magnifier *magnifier_setting)
@@ -283,6 +290,7 @@ magui_cursor_on_off_toggled (GtkWidget *widget,
 				    GCONF_VALUE_BOOL,
 				    MAGNIFIER_CURSOR,
 				    (gpointer)&magnifier_setting->cursor_state);
+	
 	magui_cursor_off (magnifier_setting->cursor_state);
     }
 }
@@ -300,6 +308,7 @@ magui_cursor_magnification_toggled (GtkWidget *widget,
 				    GCONF_VALUE_BOOL,
 				    MAGNIFIER_CURSOR_SCALE,
 				    (gpointer)&magnifier_setting->cursor_scale);
+
 	magui_cursor_magnification_off (!magnifier_setting->cursor_scale);
     }
 }
@@ -519,6 +528,7 @@ magui_crosswire_color_invert_toggled (GtkWidget *widget,
 					MAGNIFIER_CROSSWIRE_COLOR_INVERT,
 					(gpointer)&crosswire_color_old);
 	}
+	magui_crosswire_color_invert_off (magnifier_setting->crosswire_color);
     }
     else
     {
@@ -543,7 +553,9 @@ magui_crosswire_color_invert_toggled (GtkWidget *widget,
 					MAGNIFIER_CROSSWIRE_COLOR,
 					(gpointer)&magnifier_setting->crosswire_color);
 	}    
+	magui_crosswire_color_invert_off (magnifier_setting->crosswire_color);
     }    
+    
 }
 
 static void
@@ -772,6 +784,9 @@ magui_set_crosswire_color (Magnifier *magnifier_setting)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
 				     (ck_crosswire_color_invert),
 				     TRUE); 
+	gtk_widget_set_sensitive     (GTK_WIDGET
+				      (cp_crosswire_color),
+				      FALSE);				     
     }
     else
     {
@@ -794,6 +809,7 @@ magui_set_cursor (Magnifier *magnifier_setting)
     
     gtk_toggle_button_set_active (  GTK_TOGGLE_BUTTON (ck_cursor_on_off), 
 				    magnifier_setting->cursor_state);
+
     return TRUE;
 }
 
@@ -804,9 +820,10 @@ magui_set_cursor_mag (Magnifier *magnifier_setting)
     if (!w_magnifier_settings) 
 	return FALSE;
 
+    magui_cursor_magnification_off (magnifier_setting->cursor_scale);
+    
     gtk_toggle_button_set_active (  GTK_TOGGLE_BUTTON (ck_cursor_mag), 
 				    magnifier_setting->cursor_scale);
-
     return TRUE;
 }
 
@@ -819,7 +836,6 @@ magui_set_cursor_size (Magnifier *magnifier_setting)
 
     gtk_spin_button_set_value ( GTK_SPIN_BUTTON (sp_cursor_size), 
 				magnifier_setting->cursor_size);
-    
     return TRUE;
 }
 
