@@ -82,7 +82,7 @@ spui_dictionary_add_modify_set_value (void)
 
     if (spconf_dictionary_find_word (dictionary_list, word))
     {
-	gn_show_message (_("Word exist in dictionary!"));
+	gn_show_message (_("Word exists in dictionary!"));
 	return FALSE;
     }
 
