@@ -90,22 +90,12 @@ void
 magnifier_set_source_screen (GNOME_Magnifier_Magnifier 	magnifier,
 			     gchar 			*source_display_screen)
 {
-    Bonobo_PropertyBag 	properties;
-
     if (magnifier)
     {
-	properties = GNOME_Magnifier_Magnifier_getProperties (magnifier,
-				    			      &ev);
+	GNOME_Magnifier_Magnifier__set_SourceDisplay (magnifier,
+						      source_display_screen,
+						      &ev);
 	check_return_value (&ev, __LINE__);
-	if (properties && source_display_screen)
-	{
-	    bonobo_pbclient_set_string (properties,
-    			    		"source-display-screen",
-				        source_display_screen,
-				        &ev);
-	    check_return_value (&ev, __LINE__);
-	}
-	bonobo_object_release_unref (properties, NULL);
     }
 }
 
@@ -113,23 +103,13 @@ void
 magnifier_set_target_screen (GNOME_Magnifier_Magnifier 	magnifier,
 			     gchar 			*target_display_screen)
 {
-    Bonobo_PropertyBag 	properties;
-
     if (magnifier)
     {
-	properties = GNOME_Magnifier_Magnifier_getProperties (magnifier,
-			    				      &ev);
-	check_return_value (&ev, __LINE__);
-	if (properties && target_display_screen)
-	{
-	    bonobo_pbclient_set_string	(properties,
-    			    		"source-display-screen",
-					 target_display_screen,
-					 &ev);
+	    GNOME_Magnifier_Magnifier__set_TargetDisplay (magnifier,
+						    	  target_display_screen,
+						    	  &ev);
 	    check_return_value (&ev, __LINE__);
-	}
-	bonobo_object_release_unref (properties, NULL);
-  }
+    }
 }
 
 
@@ -150,7 +130,7 @@ magnifier_get_source (GNOME_Magnifier_Magnifier 	magnifier,
 	if (properties)
 	{
 	    rectangle_any = Bonobo_PropertyBag_getValue (properties,
-    			    				 "source-display-size",
+    			    				 "source-display-bounds",
   							 &ev);
 	    check_return_value (&ev, __LINE__);
 	    rectangle = (GNOME_Magnifier_RectBounds *) rectangle_any->_value;
