@@ -133,6 +133,8 @@
 #define DEFAULT_PRESENTATION_DEVICE 		"braille"
 #define DEFAULT_PRESENTATION_ROLE		"generic"
 #define DEFAULT_PRESENTATION_EVENT		"generic"
+
+#define DEFAULT_PRESENTATION_ACTIVE_SETTING_NAME 	"Default"
 /*____________________________< /PRESENTATION_DEFAULTS>___________________________*/
 /*____________________________< MAGNIFIER_DEFAULTS>___________________________*/
 #define MAX_ZOOM_FACTOR_X  			15
@@ -157,7 +159,6 @@
 #define DEFAULT_MAGNIFIER_CURSOR		TRUE
 #define DEFAULT_MAGNIFIER_CURSOR_NAME		"default"
 #define DEFAULT_MAGNIFIER_CURSOR_SIZE		32
-#define DEFAULT_MAGNIFIER_CURSOR_MAG		FALSE
 #define DEFAULT_MAGNIFIER_CURSOR_SCALE		FALSE
 #define DEFAULT_MAGNIFIER_CURSOR_COLOR		0
 
@@ -186,12 +187,16 @@
 #define DEFAULT_MAGNIFIER_TRACKING		"focus"
 #define DEFAULT_MAGNIFIER_MOUSE_TRACKING	"mouse-push"
 
+#define DEFAULT_MAGNIFIER_ALIGNMENT_X		"auto"
+#define DEFAULT_MAGNIFIER_ALIGNMENT_Y		"auto"
+
+
 #define DEFAULT_MAGNIFIER_VISIBLE		TRUE
 
 #define DEFAULT_MAGNIFIER_PANNING		FALSE	
 
-#define DEFAULT_SCREEN_MIN_SIZE			1
-#define DEFAULT_SCREEN_MAX_SIZE			1024
+#define DEFAULT_SCREEN_MIN_SIZE			0
+#define DEFAULT_SCREEN_MAX_SIZE			1023
 
 /*____________________________</MAGNIFIER_DEFAULTS>___________________________*/
 /*___________________________________________________________________________*/
@@ -252,7 +257,6 @@
 #define MAGNIFIER_CURSOR			"cursor"
 #define MAGNIFIER_CURSOR_NAME 			"cursor_name"
 #define MAGNIFIER_CURSOR_SIZE 			"cursor_size"
-#define MAGNIFIER_CURSOR_MAG    		"cursor_mag"
 #define MAGNIFIER_CURSOR_SCALE    		"cursor_scale"
 #define MAGNIFIER_CURSOR_COLOR			"cursor_color"
 
@@ -282,6 +286,9 @@
 #define MAGNIFIER_TRACKING			"tracking"
 #define MAGNIFIER_MOUSE_TRACKING		"mouse_tracking"
 
+#define MAGNIFIER_ALIGNMENT_X			"alignment_x"
+#define MAGNIFIER_ALIGNMENT_Y			"alignment_y"
+
 #define MAGNIFIER_VISIBLE 			"visible"
 #define MAGNIFIER_PANNING	  		"panning"
 /*____________________________</MAGNIFIER>____________________________________*/
@@ -323,6 +330,8 @@
 /*____________________________</COMMAND MAP>___________________________________*/
 /*____________________________<PRESENTATION>___________________________________*/
 #define PRESENTATION_CHANGES_END 		"/changes_end"
+#define PRESENTATION_SETTING_LIST		"presentation_list"
+#define PRESENTATION_ACTIVE_SETTING		"active_setting"
 /*____________________________</PRESENTATION>__________________________________*/
 /*____________________________</BRAILLE MONITOR>__________________________________*/
 #define BRAILLE_MONITOR_PANEL_POSITION_KEY 	"brlmon_position"
@@ -393,12 +402,12 @@ typedef enum {
  * types used for configuration
 **/
 typedef enum {
-    CFGT_BOOL	= 0,
-    CFGT_INT	= 1<<0,
-    CFGT_FLOAT	= 1<<1,
-    CFGT_STRING = 1<<2,
-    CFGT_LIST   = 1<<3,
-    CFGT_UNSET	= 1<<4
+    CFGT_BOOL	= GCONF_VALUE_BOOL,
+    CFGT_INT	= GCONF_VALUE_INT,
+    CFGT_FLOAT	= GCONF_VALUE_FLOAT,
+    CFGT_STRING = GCONF_VALUE_STRING,
+    CFGT_LIST   = GCONF_VALUE_LIST,
+    CFGT_UNSET	= GCONF_VALUE_INVALID
 } SRConfigTypesEnum;
 
 typedef struct {
@@ -442,7 +451,7 @@ srconf_free_slist (GSList *list);
  *	gconf_root_dir_path - the root dir of GConf
 **/
 gboolean 
-srconf_init(SRConfCB srconfcb, const gchar *gconf_root_dir_path);
+srconf_init(SRConfCB srconfcb, const gchar *_gconf_root_dir_path, const gchar *config_source);
 
 /**
  * srconf_terminate:
@@ -563,4 +572,6 @@ srconf_set_data(const gchar *key, SRConfigTypesEnum conftype, const gpointer dat
 gboolean
 srconf_unset_key (const gchar *key,const gchar *section);
 
+gchar*
+srconf_presentationi_get_chunk (const gchar *device_role_event);
 #endif /*_LIBSRCONF_H */
