@@ -30,6 +30,9 @@
 #include "srctrl.h"
 
 #undef SRCORE_PANNING_DEBUG
+#undef SRMAG_DEBUG
+
+#define SORTED_ON_COORD(a, b, c) ( (a >= 0 && a < b) ? ( (b < c) ? TRUE:FALSE):FALSE) 
 
 typedef struct
 {
@@ -57,6 +60,7 @@ typedef struct
 	gint 		 mag_zp_top	      ;
 	gint 		 mag_zp_right	      ;
 	gint 		 mag_zp_bottom	      ;
+
 	/*________</ZOOMER  PLACEMENT>_______*/
 
 	/*________< ZOOMER  PARAMETERS>______*/
@@ -72,7 +76,8 @@ typedef struct
 	gboolean	 mag_invert   ;	
 	gchar		*mag_smoothing;
 
-
+	gchar 		*mag_alignment_x   ;
+	gchar 		*mag_alignment_y   ;
 	gchar 		*mag_tracking      ;
 	gchar 		*mag_mouse_tracking;
 	
@@ -94,7 +99,9 @@ src_magnifier_send (gchar *magoutput)
     busy = TRUE;
     if (magoutput)
     {
-/*	fprintf (stderr,"\nXML:%s",magoutput);*/
+#ifdef SRMAG_DEBUG        
+	fprintf (stderr,"\nXML:%s",magoutput);
+#endif
 	mag_xml_output (magoutput, 
 			strlen (magoutput) );
     }
@@ -278,7 +285,7 @@ src_magnifier_automatic_panning (gpointer data)
 #endif
     if (stop_panning)
 	point = start_point;
-    magout = g_strdup_printf ("<MAGOUT><ZOOMER ID =\"%s\" tracking=\"%s\" AlignmentX=\"0\" AlignementY=\"0\" ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"></ZOOMER></MAGOUT>",
+    magout = g_strdup_printf ("<MAGOUT><ZOOMER ID =\"%s\" tracking=\"%s\" ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"></ZOOMER></MAGOUT>",
 				"generic_zoomer",
 				"panning",
 				point.x, point.y, point.x + 1, point.y + 1);
@@ -456,6 +463,7 @@ src_magnifier_create_mml(const Magnifier *magnifier)
 			    " CrosswireColor=\"%ld\">"
 			    " <ZOOMER ID=\"%s\" "
 			    " source=\"%s\""
+			    " target=\"%s\""
 			    " BorderWidth=\"%d\""
 			    " BorderColor=\"%ld\""
 			    " ZPLeft=\"%d\""
@@ -467,6 +475,8 @@ src_magnifier_create_mml(const Magnifier *magnifier)
 			    " invert=\"%d\""
 			    " smoothing=\"%s\""
 			    " tracking=\"%s\""
+			    " AlignmentX=\"%s\""
+			    " AlignmentY=\"%s\""
 			    " MouseTracking=\"%s\""
 			    " visible=\"%d\"></ZOOMER></MAGOUT>",
 			    magnifier->mag_cursor_state,
@@ -482,6 +492,7 @@ src_magnifier_create_mml(const Magnifier *magnifier)
 
 			    magnifier->mag_id,
 			    magnifier->mag_source,
+			    magnifier->mag_target,
 			    magnifier->mag_border_width,
 			    magnifier->mag_border_color,
 			    magnifier->mag_zp_left,
@@ -495,16 +506,13 @@ src_magnifier_create_mml(const Magnifier *magnifier)
 			    magnifier->mag_invert,
 			    magnifier->mag_smoothing,
 			    
+			    
 			    magnifier->mag_tracking,
+			    magnifier->mag_alignment_x,
+			    magnifier->mag_alignment_y,
 			    magnifier->mag_mouse_tracking,
 
 			    magnifier->mag_visible);
-/*
-    fprintf (stderr, "\n%s %d : \n%s\n",
-		    __FILE__,
-		    __LINE__,
-		    mml);
-*/
     return mml;
 }
 
@@ -565,6 +573,8 @@ src_magnifier_load_defaults ()
     magnifier->mag_invert 	   = DEFAULT_MAGNIFIER_INVERT;
     magnifier->mag_smoothing	   = g_strdup (DEFAULT_MAGNIFIER_SMOOTHING);
 
+    magnifier->mag_alignment_x 	   = g_strdup (DEFAULT_MAGNIFIER_ALIGNMENT_X);
+    magnifier->mag_alignment_y 	   = g_strdup (DEFAULT_MAGNIFIER_ALIGNMENT_Y);
     magnifier->mag_tracking 	   = g_strdup (DEFAULT_MAGNIFIER_TRACKING);
     magnifier->mag_mouse_tracking  = g_strdup (DEFAULT_MAGNIFIER_MOUSE_TRACKING);
 
@@ -581,81 +591,74 @@ src_magnifier_load_values ()
     gboolean   default_bool_val;
     gint       default_int_val;
 /*    gchar     *default_string_val = NULL;*/
+    gint 	mag_size_x,
+		mag_size_y;
 
     /*_________________________< GENERAL PROPERTIES>__________________________*/
     /*_______________________________< CURSOR>________________________________*/
     default_bool_val = DEFAULT_MAGNIFIER_CURSOR;
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+    srconf_get_data_with_default (MAGNIFIER_CURSOR, 
 				       CFGT_BOOL, 
 				       (gpointer)&magnifier->mag_cursor_state, 
 				       (gpointer)&default_bool_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-    
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_NAME, 
+				       MAGNIFIER_CONFIG_PATH);
+
+    srconf_get_data_with_default (MAGNIFIER_CURSOR_NAME, 
 				      CFGT_STRING, 
 				      &magnifier->mag_cursor_name, 
 				      (gpointer)DEFAULT_MAGNIFIER_CURSOR_NAME, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				      MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_CURSOR_SIZE;
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_SIZE, 
+    srconf_get_data_with_default (MAGNIFIER_CURSOR_SIZE, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_cursor_size, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_bool_val = DEFAULT_MAGNIFIER_CURSOR_SCALE;
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_SCALE, 
+    srconf_get_data_with_default (MAGNIFIER_CURSOR_SCALE, 
 				       CFGT_BOOL, 
 				       (gpointer)&magnifier->mag_cursor_scale, 
 				       (gpointer)&default_bool_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-
+				       MAGNIFIER_CONFIG_PATH);
+		    
     default_int_val = DEFAULT_MAGNIFIER_CURSOR_COLOR;
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_COLOR, 
+    srconf_get_data_with_default (MAGNIFIER_CURSOR_COLOR, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_cursor_color, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
     /*______________________________</CURSOR>_________________________________*/
 
     /*_____________________________< CROSSWIRE>_______________________________*/
     default_int_val = DEFAULT_MAGNIFIER_CROSSWIRE;
-    if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
+    srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
 				       CFGT_BOOL, 
 				       (gpointer)&magnifier->mag_crosswire_state, 
 				       (gpointer)&default_bool_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_bool_val = DEFAULT_MAGNIFIER_CROSSWIRE_CLIP;
-    if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_CLIP, 
+    srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_CLIP, 
 				       CFGT_BOOL, 
 				       (gpointer)&magnifier->mag_crosswire_clip, 
 				       (gpointer)&default_bool_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_CROSSWIRE_SIZE;
-    if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_SIZE, 
+    srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_SIZE, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_crosswire_size, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_CROSSWIRE_COLOR;
-    if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_COLOR, 
+    srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_COLOR, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_crosswire_color, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
     /*______________________________</CROSSWIRE>______________________________*/
     /*__________________________</GENERAL PROPERTIES>_________________________*/
 
@@ -663,52 +666,93 @@ src_magnifier_load_values ()
     /*__________________________< ZOOMER PROPERTIES>__________________________*/
     /*__________________________< ZOOMER  PLACEMENT>__________________________*/
     default_int_val = DEFAULT_MAGNIFIER_BORDER_WIDTH;
-    if (!srconf_get_data_with_default (MAGNIFIER_BORDER_WIDTH, 
+    srconf_get_data_with_default (MAGNIFIER_BORDER_WIDTH, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_border_width, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_BORDER_COLOR;
-    if (!srconf_get_data_with_default (MAGNIFIER_BORDER_COLOR, 
+    srconf_get_data_with_default (MAGNIFIER_BORDER_COLOR, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_border_color, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-	
+				       MAGNIFIER_CONFIG_PATH);
+
+    default_int_val = DEFAULT_MAGNIFIER_DISPLAY_SIZE_X;
+    srconf_get_data_with_default (MAGNIFIER_DISPLAY_SIZE_X, 
+				       CFGT_INT, 
+				       (gpointer)&mag_size_x, 
+				       (gpointer)&default_int_val, 
+				       MAGNIFIER_CONFIG_PATH);
+
+    default_int_val = DEFAULT_MAGNIFIER_DISPLAY_SIZE_Y;
+    srconf_get_data_with_default (MAGNIFIER_DISPLAY_SIZE_Y, 
+				       CFGT_INT, 
+				       (gpointer)&mag_size_y, 
+				       (gpointer)&default_int_val, 
+				       MAGNIFIER_CONFIG_PATH);
+
     default_int_val = DEFAULT_MAGNIFIER_ZP_LEFT;
-    if (!srconf_get_data_with_default (MAGNIFIER_ZP_LEFT, 
+    srconf_get_data_with_default (MAGNIFIER_ZP_LEFT, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_zp_left, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_ZP_TOP;
-    if (!srconf_get_data_with_default (MAGNIFIER_ZP_TOP, 
+    srconf_get_data_with_default (MAGNIFIER_ZP_TOP, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_zp_top, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_ZP_WIDTH;
-    if (!srconf_get_data_with_default (MAGNIFIER_ZP_WIDTH, 
+    srconf_get_data_with_default (MAGNIFIER_ZP_WIDTH, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_zp_right, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_ZP_HEIGHT;
-    if (!srconf_get_data_with_default (MAGNIFIER_ZP_HEIGHT, 
+    srconf_get_data_with_default (MAGNIFIER_ZP_HEIGHT, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_zp_bottom, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				       MAGNIFIER_CONFIG_PATH);
+    /* do a vertical split*/
+     
+    if (!SORTED_ON_COORD (magnifier->mag_zp_left,
+			 magnifier->mag_zp_right,
+			 mag_size_x) )
+    {
+	magnifier->mag_zp_left = mag_size_x/2;
+	magnifier->mag_zp_right = mag_size_x - 1;
+	srconf_set_data (MAGNIFIER_ZP_LEFT, 
+		         CFGT_INT, 
+			 (gpointer)&magnifier->mag_zp_left, 
+  		         MAGNIFIER_CONFIG_PATH);
+	srconf_set_data (MAGNIFIER_ZP_WIDTH, 
+		         CFGT_INT, 
+			 (gpointer)&magnifier->mag_zp_right, 
+  		         MAGNIFIER_CONFIG_PATH);
+    }			 
+
+    if (!SORTED_ON_COORD (magnifier->mag_zp_top,
+			 magnifier->mag_zp_bottom,
+			 mag_size_y) )
+    {
+	magnifier->mag_zp_top = 0;
+	magnifier->mag_zp_bottom = mag_size_y - 1;
+	srconf_set_data (MAGNIFIER_ZP_TOP, 
+		         CFGT_INT, 
+			 (gpointer)&magnifier->mag_zp_top, 
+  		         MAGNIFIER_CONFIG_PATH);
+	srconf_set_data (MAGNIFIER_ZP_HEIGHT, 
+		         CFGT_INT, 
+			 (gpointer)&magnifier->mag_zp_bottom, 
+  		         MAGNIFIER_CONFIG_PATH);
+    }			 
 
     /*__________________________</ZOOMER  PLACEMENT>__________________________*/
 
@@ -722,83 +766,83 @@ src_magnifier_load_values ()
 					 MAGNIFIER_CONFIG_PATH)
 	 ) return NULL;
 */
-    if (!srconf_get_data_with_default (MAGNIFIER_SOURCE, 
+    srconf_get_data_with_default (MAGNIFIER_SOURCE, 
 				      CFGT_STRING, 
 				      &magnifier->mag_source, 
 				      (gpointer)DEFAULT_MAGNIFIER_SOURCE, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-    if (!srconf_get_data_with_default (MAGNIFIER_TARGET, 
+				      MAGNIFIER_CONFIG_PATH);
+    
+    srconf_get_data_with_default (MAGNIFIER_TARGET, 
 				      CFGT_STRING, 
 				      &magnifier->mag_target, 
 				      (gpointer)DEFAULT_MAGNIFIER_TARGET, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-
-
+				      MAGNIFIER_CONFIG_PATH);
 
     default_int_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
+    srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_zoom_factor_x, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
+				       MAGNIFIER_CONFIG_PATH);
+
+    srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
 				       CFGT_INT, 
 				       (gpointer)&magnifier->mag_zoom_factor_y, 
 				       (gpointer)&default_int_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-
+				       MAGNIFIER_CONFIG_PATH);
 /*    magnifier->mag_zoom_factor_lock = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;*/
 
 
     default_bool_val = DEFAULT_MAGNIFIER_INVERT;
-    if (!srconf_get_data_with_default (MAGNIFIER_INVERT, 
+    srconf_get_data_with_default (MAGNIFIER_INVERT, 
 				      CFGT_BOOL, 
 				      (gpointer)&magnifier->mag_invert, 
 				      (gpointer)&default_bool_val, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-
-    if (!srconf_get_data_with_default (MAGNIFIER_SMOOTHING, 
+				      MAGNIFIER_CONFIG_PATH);
+    
+    srconf_get_data_with_default (MAGNIFIER_SMOOTHING, 
 				      CFGT_STRING, 
 				      &magnifier->mag_smoothing, 
 				      (gpointer)DEFAULT_MAGNIFIER_SMOOTHING, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
+				      MAGNIFIER_CONFIG_PATH);
 
-    if (!srconf_get_data_with_default (MAGNIFIER_TRACKING, 
+    srconf_get_data_with_default (MAGNIFIER_ALIGNMENT_X, 
+				      CFGT_STRING, 
+				      &magnifier->mag_alignment_x, 
+				      (gpointer)DEFAULT_MAGNIFIER_ALIGNMENT_X, 
+				      MAGNIFIER_CONFIG_PATH);
+
+    srconf_get_data_with_default (MAGNIFIER_ALIGNMENT_Y, 
+				      CFGT_STRING, 
+				      &magnifier->mag_alignment_y, 
+				      (gpointer)DEFAULT_MAGNIFIER_ALIGNMENT_Y, 
+				      MAGNIFIER_CONFIG_PATH);
+
+    srconf_get_data_with_default (MAGNIFIER_TRACKING, 
 				      CFGT_STRING, 
 				      &magnifier->mag_tracking, 
 				      (gpointer)DEFAULT_MAGNIFIER_TRACKING, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-    if (!srconf_get_data_with_default (MAGNIFIER_MOUSE_TRACKING, 
+				      MAGNIFIER_CONFIG_PATH);
+
+    srconf_get_data_with_default (MAGNIFIER_MOUSE_TRACKING, 
 				      CFGT_STRING, 
 				      &magnifier->mag_mouse_tracking, 
 				      (gpointer)DEFAULT_MAGNIFIER_MOUSE_TRACKING, 
-				      MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-
+				      MAGNIFIER_CONFIG_PATH);
 
     default_bool_val = DEFAULT_MAGNIFIER_VISIBLE;
-    if (!srconf_get_data_with_default (MAGNIFIER_VISIBLE, 
+    srconf_get_data_with_default (MAGNIFIER_VISIBLE, 
 				       CFGT_BOOL, 
 				       (gpointer)&magnifier->mag_visible, 
 				       (gpointer)&default_bool_val, 
-				       MAGNIFIER_CONFIG_PATH)
-	) return NULL;
-	
+				       MAGNIFIER_CONFIG_PATH);
+
     default_bool_val = DEFAULT_MAGNIFIER_PANNING;
-    if (!srconf_get_data_with_default ( MAGNIFIER_PANNING,
+    srconf_get_data_with_default ( MAGNIFIER_PANNING,
 				        CFGT_BOOL, 
 					(gpointer)&panning, 
 					(gpointer)&default_bool_val, 
-					MAGNIFIER_CONFIG_PATH)
-	) return NULL; 	
-
+					MAGNIFIER_CONFIG_PATH);
     /*_________________________</ZOOMER  PARAMETERS>__________________________*/
     /*_________________________</ZOOMER  PROPERTIES>__________________________*/
 
@@ -848,6 +892,17 @@ src_magnifier_setting_free (Magnifier* magnifier)
 	magnifier->mag_smoothing = NULL;
     }
 
+    if (magnifier->mag_alignment_x)
+    {
+	g_free (magnifier->mag_alignment_x);
+	magnifier->mag_alignment_x = NULL;
+    }
+
+    if (magnifier->mag_alignment_y)
+    {
+	g_free (magnifier->mag_alignment_y);
+	magnifier->mag_alignment_y = NULL;
+    }
 
     if (magnifier->mag_tracking)
     {
@@ -892,6 +947,19 @@ src_magnifier_config_set_defaults (Magnifier *magnifier)
 			  (gpointer)magnifier->mag_mouse_tracking,
 			  MAGNIFIER_CONFIG_PATH)
 	) return FALSE;	
+
+    if (!srconf_set_data (MAGNIFIER_ALIGNMENT_X, 
+			  CFGT_STRING, 
+			  (gpointer)magnifier->mag_alignment_x, 
+			  MAGNIFIER_CONFIG_PATH)
+	) return FALSE;
+
+    if (!srconf_set_data (MAGNIFIER_ALIGNMENT_Y, 
+			  CFGT_STRING, 
+			  (gpointer)magnifier->mag_alignment_y, 
+			  MAGNIFIER_CONFIG_PATH)
+	) return FALSE;
+
     if (!srconf_set_data (MAGNIFIER_TRACKING, 
 			  CFGT_STRING, 
 			  (gpointer)magnifier->mag_tracking, 
