@@ -89,7 +89,178 @@ static gboolean src_process_space 	(gchar *space);
 static gboolean src_process_modifier 	(gchar *modifier);
 static gboolean src_process_cursor 	(gchar *cursor);
 
+static void
+src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
+{
+    gchar *txt;
 
+    srl_assert (src_use_braille || src_use_braille_monitor);
+
+	
+    src_enable_format_string = FALSE; 
+    txt = src_presentation_sro_get_pres_for_device (obj, reason, "braille");
+    if (txt)
+    {
+	gchar *tmp;
+	gchar *cursor = NULL;
+	SRObjectRoles role;
+	gchar *status = NULL;
+	gchar offset[5];
+
+	sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+	if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL)
+	{
+	    SRLong offset;
+	    SRTextAttribute *attr;
+	    gchar status_[5] = "    ";
+
+	    if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
+		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset, src_braille_cursor_style);
+	    
+	    if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
+				    &attr, SR_INDEX_CONTAINER))
+	    {
+		gchar *tmp;
+		if (sra_get_attribute_value (attr[0], "bold", &tmp))
+		{
+		    if (strcmp (tmp, "true") == 0)
+			status_[0] = 'B';
+		    SR_freeString (tmp);
+		}
+	    	if (sra_get_attribute_value (attr[0], "italic", &tmp))
+		{
+		    if (strcmp (tmp, "true") == 0)
+		        status_[1] = 'I';
+		    SR_freeString (tmp);
+		}
+	    	if (sra_get_attribute_value (attr[0], "underline", &tmp))
+		{
+		    if (strcmp (tmp, "single") == 0 || strcmp (tmp, "double") == 0)
+		        status_[2] = 'U';
+		    SR_freeString (tmp);
+		}
+	    	if (sra_get_attribute_value (attr[0], "selected", &tmp))
+		{
+		    if (strcmp (tmp, "true") == 0)
+		        status_[3] = 'S';
+		    SR_freeString (tmp);
+		}
+		sra_free (attr);
+		status = g_strconcat("<BRLDISP role=\"status\" start=\"0\" clear=\"true\">"
+					 "<TEXT>",
+					 status_,
+					 "</TEXT>"
+					 "</BRLDISP>", 
+					 NULL);
+	    }
+	}
+	sprintf (offset, "%d", src_braille_offset);
+	tmp = g_strconcat ("<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">",
+			"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\"",
+			cursor ? cursor : "",
+			">",
+			txt,
+			"</BRLDISP>",
+			status ? status : "",
+			"</BRLOUT>",
+			NULL);
+	/*fprintf (stderr, "\n%s", tmp);*/
+	if (src_use_braille)
+	    src_braille_send (tmp);
+	if (src_use_braille_monitor)
+	    src_brlmon_send (tmp);
+	g_free (tmp);
+	g_free (cursor);
+    }
+    g_free (txt);
+}
+
+
+static void
+src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
+{
+    SRObjectRoles role;
+
+    srl_assert (src_use_speech);
+
+    src_enable_format_string = TRUE; 
+    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+    if (strcmp (reason, "object:text-changed:insert") == 0
+	    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL))
+    {
+	gchar *diff;
+	if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
+	{
+	    if (diff[1] != '\0')
+	    {
+		if (src_last_key_echo)
+		{
+		    g_free (src_last_key_echo);
+		    src_last_key_echo = NULL;
+		}
+		src_last_key_echo = diff;
+		src_cmd_queue_add ("kbd key");
+		src_cmd_queue_process ();
+		src_last_key_echo = NULL;
+	    }
+	    else
+	    {
+		if (g_ascii_ispunct (diff[0]))
+    		    src_process_punct (diff);
+		else if (g_ascii_isspace (diff[0]))
+    		    src_process_space (diff);
+		else
+    		    src_process_key_ (diff);
+	    }
+	    SR_freeString (diff);
+	}
+    }
+    else
+    {
+	gchar *txt;
+    	if (src_last_key_echo)
+	{
+	    g_free (src_last_key_echo);
+	    src_last_key_echo = NULL;
+	}
+	txt = src_presentation_sro_get_pres_for_device (obj, reason, "speech");
+	if (txt && txt[0])
+	{
+	    SRObjectRoles role;
+	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+	    if ( role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
+	    else if ((role == SR_ROLE_TABLE || role == SR_ROLE_TREE_TABLE) &&
+		strcmp (reason , "focus:") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
+	    else
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
+	}
+	g_free (txt);
+    }
+}
+
+static void
+src_present_real_sro_for_magnifier (SRObject *obj, gchar *reason)
+{
+    gchar *txt;
+
+    srl_assert (src_use_magnifier);
+
+    txt = src_presentation_sro_get_pres_for_device (obj, reason, "magnifier");
+    if (txt)
+    {
+        gchar *tmp;
+        tmp = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" tracking=\"focus\" ",
+			txt,
+			"></ZOOMER></MAGOUT>",
+			NULL);
+        if (tmp)
+	    src_magnifier_send (tmp);
+	g_free (tmp);
+	g_free (txt);
+    }
+}
 
 void
 src_present_real_sro (SRObject *obj)
@@ -108,172 +279,16 @@ src_present_real_sro (SRObject *obj)
     reason = NULL;
     if (!sro_get_reason (obj, &reason))
 	return;
-    
+
     if (src_use_braille || src_use_braille_monitor)
-    {
-	gchar *txt;
-	
-	src_enable_format_string = FALSE; 
-    	txt = src_presentation_present_sro_for_device (obj, reason, "braille");
-	if (txt)
-	{
-	    gchar *tmp;
-	    gchar *cursor = NULL;
-	    SRObjectRoles role;
-	    gchar *status = NULL;
-	    gchar offset[5];
-
-	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if (role == SR_ROLE_TEXT || role == SR_ROLE_TERMINAL)
-	    {
-		SRLong offset;
-		SRTextAttribute *attr;
-		gchar status_[5] = "    ";
-
-		if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
-		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset + 4, src_braille_cursor_style);
-	    
-		if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
-				    &attr, SR_INDEX_CONTAINER))
-		{
-		    gchar *tmp;
-		    if (sra_get_attribute_value (attr[0], "bold", &tmp))
-		    {
-			if (strcmp (tmp, "true") == 0)
-			    status_[0] = 'B';
-			SR_freeString (tmp);
-		    }
-	    	    if (sra_get_attribute_value (attr[0], "italic", &tmp))
-		    {
-			if (strcmp (tmp, "true") == 0)
-			    status_[1] = 'I';
-			SR_freeString (tmp);
-		    }
-	    	    if (sra_get_attribute_value (attr[0], "underline", &tmp))
-		    {
-			if (strcmp (tmp, "single") == 0 || strcmp (tmp, "double") == 0)
-			    status_[2] = 'U';
-			SR_freeString (tmp);
-		    }
-	    	    if (sra_get_attribute_value (attr[0], "selected", &tmp))
-		    {
-			if (strcmp (tmp, "true") == 0)
-			    status_[3] = 'S';
-			SR_freeString (tmp);
-		    }
-		    sra_free (attr);
-		    status = g_strconcat("<BRLDISP role=\"status\" start=\"0\" clear=\"true\">"
-					 "<TEXT>",
-					 status_,
-					 "</TEXT>"
-					 "</BRLDISP>", 
-					 NULL);
-		}
-	    }
-	    sprintf (offset, "%d", src_braille_offset);
-	    tmp = g_strconcat ("<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">",
-				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\"",
-				cursor ? cursor : "",
-				">",
-				txt,
-				"</BRLDISP>",
-				status ? status : "",
-				"</BRLOUT>",
-				NULL);
-	    /*fprintf (stderr, "\n%s", tmp);*/
-	    if (src_use_braille)
-		src_braille_send (tmp);
-	    if (src_use_braille_monitor)
-		src_brlmon_send (tmp);
-	    g_free (tmp);
-	    g_free (cursor);
-	}
-	g_free (txt);
-    }
+	src_present_real_sro_for_brl_and_brlmon (obj, reason);
     
     if (src_use_speech)
-    {
-	src_enable_format_string = TRUE; 
-	{
-	    SRObjectRoles role;
-	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if (strcmp (reason, "object:text-changed:insert") == 0
-		    && (role == SR_ROLE_TEXT || role == SR_ROLE_TERMINAL))
-	    {
-		gchar *diff;
-		if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
-		{
-		    if (diff[1] != '\0')
-		    {
-			if (src_last_key_echo)
-			{
-			    g_free (src_last_key_echo);
-			    src_last_key_echo = NULL;
-			}
-			src_last_key_echo = diff;
-			src_cmd_queue_add ("kbd key");
-			src_cmd_queue_process ();
-			src_last_key_echo = NULL;
-		    }
-		    else
-		    {
-			if (g_ascii_ispunct (diff[0]))
-	    		    src_process_punct (diff);
-			else if (g_ascii_isspace (diff[0]))
-	    		    src_process_space (diff);
-			else
-	    		    src_process_key_ (diff);
-		    }
-		    SR_freeString (diff);
-		}
-	    }
-	    else
-	    {
-		gchar *txt;
-
-	    	if (src_last_key_echo)
-		{
-		    g_free (src_last_key_echo);
-		    src_last_key_echo = NULL;
-		}
-		txt = src_presentation_present_sro_for_device (obj, reason, "speech");
-    		if (txt && txt[0])
-    		{
-		    SRObjectRoles role;
-		    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-		    if (role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
-			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
-		    else
-			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
-		}
-		g_free (txt);
-	    }
-	}
-    }
-        
+	src_present_real_sro_for_speech (obj, reason);
 
     if (src_use_magnifier)
-    {
-	gchar *txt;
-	txt = src_presentation_present_sro_for_device (obj, reason, "magnifier");
-	if (txt)
-	{
-/*	    fprintf (stderr, "\n(%s %d) txt = %s",
-			__FILE__,
-			__LINE__,
-			txt);
-*/
-	    gchar *tmp;
-	    tmp = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" tracking=\"focus\" ",
-				txt,
-				"></ZOOMER></MAGOUT>",
-				NULL);
-	    if (tmp)
-		src_magnifier_send (tmp);
-	    g_free (tmp);
-	    g_free (txt);
-	}
-    }
+	src_present_real_sro_for_magnifier (obj, reason);
+
     SR_freeString (reason);
 }
 
@@ -310,7 +325,7 @@ src_present_crt_window ()
     	gchar *txt;
 
 	src_enable_format_string = FALSE; 
-	txt = src_presentation_present_sro_for_device (src_crt_window, reason, "speech");
+	txt = src_presentation_sro_get_pres_for_device (src_crt_window, reason, "speech");
     	if (txt && txt[0])
     	{
 	    if (strcmp (reason, "window:create") == 0)
@@ -339,7 +354,7 @@ src_present_crt_tooltip ()
     	gchar *txt;
 
 	src_enable_format_string = FALSE; 
-	txt = src_presentation_present_sro_for_device (src_crt_tooltip, reason, "speech");
+	txt = src_presentation_sro_get_pres_for_device (src_crt_tooltip, reason, "speech");
     	if (txt && txt[0])
     	{
 	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
@@ -1351,12 +1366,47 @@ static gboolean src_crosswire	    	= FALSE;
     {
     }
 
-    else
+    else*/
     if (config->key && 
 	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_TARGET) == 0)
     {
+	if (config->newvalue &&
+	    config->type == CFGT_STRING)
+	{
+	    gchar *target = (gchar*)config->newvalue;
+
+	    if (src_use_magnifier)
+	    {
+		gchar *mag_out;
+
+		if (src_use_speech)
+		{
+		    gchar *tmp;
+		    tmp = g_strdup_printf ("%s %s",
+					   _("magnifier target"),
+					   target);
+		    src_say_message (tmp);
+		    g_free (tmp);
+		}
+
+		mag_out = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" target=\"",
+			    		target,
+					" \"></ZOOMER></MAGOUT>",
+					NULL);
+		if (mag_out)
+		{
+		 /*   fprintf (stderr, "\n%s %d : target %s",
+					__FILE__,
+					__LINE__,
+					target);
+		 */ src_magnifier_send (mag_out);
+		    g_free (mag_out);
+		    mag_out = NULL;
+		}
+	    }
+	}
     }
-    else*/
+    else
       if (config->key &&
 	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_SOURCE) == 0)
     {
@@ -1685,103 +1735,27 @@ src_process_config_changed_for_control (SRConfigStructure *config)
     /*src_update_key ();*/
 }
 
-static xmlNode *
-load_presentation_file (gchar *fname) 
-{
-    xmlDoc *doc;
-    xmlNode *rootnode;
-    if ((doc = xmlParseFile (fname)) && (rootnode = xmlDocGetRootElement (doc))) 
-    {
-	if (xmlSearchNsByHref (doc, rootnode, (const xmlChar *) "http://www.gnome.org/gnopernicus") != NULL)
-	{
-	    if (xmlStrcmp (rootnode->name, (const xmlChar *) "PresentationProfile") == 0) 
-	    {
-		return rootnode;
-	    }
-	    else
-	    {
-		fprintf (stderr, "Root element is not a PresentationProfile\n.");
-	    }
-	}
-	else
-	{
-	    fprintf (stderr, "Error: incorrect namespace in presentation file.\n");	    
-	}
-    }
-    else 
-    {
-	fprintf (stderr, doc ? "file %s not found\n" : "file %s is empty.\n", fname ? fname : "<nil>" );
-	xmlFreeDoc (doc);
-	return NULL;
-    }
-    return NULL;
-}
-
 static void
-set_presentation_gconf_key (gchar *key, gchar *value)
+src_process_change_for_presentation (gboolean first)
 {
-    srconf_set_data (key, CFGT_STRING, value, PRESENTATION_PATH);
+    gchar *name = "default";
+    gchar *name2;
+	
+    srconf_get_data_with_default (PRESENTATION_ACTIVE_SETTING, CFGT_STRING, &name, &name, PRESENTATION_PATH);
+
+    name2 = g_strconcat (PRESENTATION_DIR, name, ".xml", NULL); 
+    if (!first)
+	src_presentation_terminate ();
+    src_presentation_init (name2);
+    g_free (name2);
 }
 
-static gint
-set_gconf_keys_from_xml (xmlNode *root)
-{
-    gboolean found = FALSE;
-    xmlNode *node;
-    const GList *lang_list = gnome_i18n_get_language_list ("LC_MESSAGES");
-
-    while (!found && lang_list) 
-    {
-	for (node = root->xmlChildrenNode; node != NULL; node = node->next)
-	{
-	    if (xmlStrcmp (node->name, (const xmlChar *) "_presentation-key") == 0) 
-	    {
-		char *lang = xmlNodeGetLang (node);
-		char *locale = lang ? lang : "C";
-		if (!xmlStrcmp (lang_list->data, locale))
-		{
-		    gchar *key;
-		    gchar *value;
-		    found = TRUE;
-		    if (lang) xmlFree (lang);
-		    key = xmlGetProp (node, (const xmlChar *) "type");
-		    value = xmlNodeGetContent (node);
-		    if (key && value) 
-		    {
-			set_presentation_gconf_key (key, value);
-			xmlFree (value);
-		    }
-		}
-	    }
-	}
-	lang_list = lang_list->next;
-    }
-
-    return !found;
-}
-
-
 static void
 src_process_config_changed_for_presentation (SRConfigStructure *config)
 {
     if (config)
     {
-	xmlNode *root;
-	gchar *name = "default";
-	gchar *name2;
-	
-	srconf_get_data_with_default (PRESENTATION_ACTIVE_SETTING, CFGT_STRING, &name, &name, PRESENTATION_PATH);
-
-	name2 = g_strconcat (PRESENTATION_DIR, name, ".xml", NULL); 
-	root = load_presentation_file (name2);
-	g_free (name);
-	g_free (name2);
-
-	if (root)
-	    set_gconf_keys_from_xml (root);
-
-	src_presentation_terminate ();
-	src_presentation_init ();
+	src_process_change_for_presentation (FALSE);
     }
 }
 
@@ -2191,7 +2165,10 @@ src_process_key (SREvent *event,
 
     if (!srhotkey_data->keystring)
 	return;
-    
+    if (strcmp (srhotkey_data->keystring, "Control_L") == 0)
+	if (src_use_speech)
+	    src_speech_shutup ();
+
     if (strncmp (srhotkey_data->keystring, "Alt", 3) == 0 ||
 	strncmp (srhotkey_data->keystring, "Control", 7) == 0 ||
 	strncmp (srhotkey_data->keystring, "Shift", 5) == 0 ||
@@ -2318,18 +2295,20 @@ src_process_sro_event (SREvent *event,
 
     if (sre_get_event_data (event, (void**)&obj))
     {
+        if (src_use_magnifier)
+	{
+	    gchar *reason;
+	    if (sro_get_reason (obj, &reason))
+	    {
+		if (strcmp (reason, "object:visible-data-changed") == 0)
+		    src_present_real_sro_for_magnifier (obj, reason);
+		SR_freeString (reason);
+	    }	    
+	}
 	src_process_sro (obj);
 	if (src_use_magnifier)
 	    src_magnifier_start_panning (obj);
     }
-/*
-    {
-	gchar *r;
-	sro_get_reason (obj, &r);
-	fprintf (stderr, "\n%s", r);
-	SR_freeString (r);
-    }
-*/
 }
 
 static void
@@ -2731,6 +2710,7 @@ src_init(const gchar *mouse_tracking,
     gchar *message = _("Welcome to Gnopernicus");
     gboolean brl_error = FALSE;
     gboolean initialize_finished = FALSE;
+    gboolean default_invert_value;
 
     /* initialize SR Utils */
     sru_init();
@@ -2757,7 +2737,18 @@ src_init(const gchar *mouse_tracking,
     /* initialize Speech */
     if (src_use_speech)
 	src_use_speech = src_speech_init ();
-
+    
+    /* check if invert is true */
+    if (!invert)
+    {
+	default_invert_value = DEFAULT_MAGNIFIER_INVERT;
+	srconf_get_data_with_default (MAGNIFIER_INVERT,
+				      CFGT_BOOL,
+				      (gpointer)&invert,
+				      (gpointer)&default_invert_value,
+				      MAGNIFIER_CONFIG_PATH);
+    }
+    
     /* set the command lines arguments in gconf */
     src_set_command_lines (mouse_tracking,
 		    	    focus_tracking,
@@ -2772,7 +2763,8 @@ src_init(const gchar *mouse_tracking,
 	    src_magnifier_create ();
     }
     
-    src_presentation_init ();	
+    src_process_change_for_presentation (TRUE);
+
     /* initialize SRLow */
     srl_init();
 
