@@ -88,16 +88,16 @@ typedef enum
 
 typedef enum
 {
-    SRC_SPEECH_PRIORITY_ERROR,
-    SRC_SPEECH_PRIORITY_WARNING,
-    SRC_SPEECH_PRIORITY_SYSTEM,
-    SRC_SPEECH_PRIORITY_MESSAGE,
     SRC_SPEECH_PRIORITY_IDLE,
+    SRC_SPEECH_PRIORITY_MESSAGE,
+    SRC_SPEECH_PRIORITY_SYSTEM,
+    SRC_SPEECH_PRIORITY_WARNING,
+    SRC_SPEECH_PRIORITY_ERROR,
 }SRCSpeechPriority;
 
 gboolean src_speech_init	  ();
 void     src_speech_terminate	  ();
-gboolean src_speech_send_chunk 	  (gchar *chunk, SRCSpeechPriority priority);
+gboolean src_speech_send_chunk 	  (gchar *chunk, SRCSpeechPriority priority, gboolean preempt);
 void     src_speech_shutup 	  ();
 void     src_speech_pause 	  ();
 void     src_speech_resume 	  ();
