@@ -68,6 +68,7 @@
 #define SRL_EVENT_NAME_UNKNOWN			"unknown"
 #define SRL_EVENT_NAME_ACTIVE_DESCENDANT_CHANGED "object:active-descendant-changed"
 #define SRL_EVENT_NAME_NAME_CHANGED		"object:property-change:accessible-name"
+#define SRL_EVENT_NAME_VISIBLE_DATA_CHANGED	"object:visible-data-changed"
 
 #define SRL_TOOLKIT_MOZILLA			"mozilla"
 #define SRL_APP_METACITY			"metacity"
@@ -79,6 +80,8 @@ typedef enum _SRLEventType
     SRL_EVENT_FOCUS1, /* some events are reported as "focus:",
 			but all other events for those objects 
 			should not be reported. (eg ALT+CTRL+TAB) */
+    SRL_EVENT_FOCUS2, /* some text object are not focused, but user can 
+			interact with them */
     SRL_EVENT_LINK_SELECTED,
     SRL_EVENT_TEXT_CARET_MOVED,
     SRL_EVENT_TEXT_CHANGED_INSERT,
@@ -103,6 +106,7 @@ typedef enum _SRLEventType
     SRL_EVENT_TOOLTIP_HIDE,
     SRL_EVENT_ACTIVE_DESCENDANT_CHANGED,
     SRL_EVENT_NAME_CHANGED,
+    SRL_EVENT_VISIBLE_DATA_CHANGED,
     SRL_EVENT_WINDOW_RENAME
 }SRLEventType;
 
@@ -145,7 +149,8 @@ static SRLEventTypeEventName srl_events_type_name[] =
 	{SRL_EVENT_WINDOW_DEACTIVATE,		SRL_EVENT_NAME_WINDOW_DEACTIVATE},
 	{SRL_EVENT_MOUSE_ABS,			SRL_EVENT_NAME_MOUSE_ABS},
 	{SRL_EVENT_ACTIVE_DESCENDANT_CHANGED, 	SRL_EVENT_NAME_ACTIVE_DESCENDANT_CHANGED},
-	{SRL_EVENT_NAME_CHANGED,		SRL_EVENT_NAME_NAME_CHANGED}
+	{SRL_EVENT_NAME_CHANGED,		SRL_EVENT_NAME_NAME_CHANGED},
+	{SRL_EVENT_VISIBLE_DATA_CHANGED,	SRL_EVENT_NAME_VISIBLE_DATA_CHANGED}
     };
 
 typedef struct _SRLEvent
@@ -184,6 +189,8 @@ extern gboolean sro_set_name (SRObject *obj, gchar *name);
 extern gboolean sro_get_from_accessible_event (Accessible *acc, gchar *reason, 
 						SRObject **obj);
 extern Accessible* sro_get_acc (SRObject *obj);
+extern gint sr_acc_get_link_index (Accessible *acc);
+
 #define sro_get_from_acc	sro_get_from_accessible_event
 
 #ifdef SRL_TIMING
@@ -678,6 +685,8 @@ srle_get_reason (SRLEvent *event)
 	return SRL_EVENT_NAME_WINDOW_RENAME;
     if (srle_has_type (event, SRL_EVENT_FOCUS1))
 	return SRL_EVENT_NAME_FOCUS;
+    if (srle_has_type (event, SRL_EVENT_FOCUS2))
+	return SRL_EVENT_NAME_FOCUS;
 
     srl_assert_not_reached ();
     return SRL_EVENT_NAME_UNKNOWN;
@@ -688,6 +697,8 @@ srl_is_window_event (SRLEvent *event)
 {
     srl_assert (event);
     
+    if (srle_acc_has_role (event, SPI_ROLE_TOOL_TIP))
+	return FALSE;
     return  event->type == SRL_EVENT_WINDOW_CREATE   ||
 	    event->type == SRL_EVENT_WINDOW_DESTROY  ||
 	    event->type == SRL_EVENT_WINDOW_MINIMIZE ||
@@ -706,6 +717,7 @@ srl_is_focus_event (SRLEvent *event)
     
     return  event->type == SRL_EVENT_FOCUS		||
 	    event->type == SRL_EVENT_FOCUS1		||
+	    event->type == SRL_EVENT_FOCUS2		||
 	    event->type == SRL_EVENT_LINK_SELECTED	||
 	    event->type == SRL_EVENT_TEXT_CARET_MOVED	||
 	    event->type == SRL_EVENT_TEXT_CHANGED_INSERT||
@@ -717,7 +729,8 @@ srl_is_focus_event (SRLEvent *event)
 	    event->type == SRL_EVENT_STATE_SELECTED	||
 	    event->type == SRL_EVENT_STATE_EXPANDED	||
 	    event->type == SRL_EVENT_STATE_SHOWING	||
-	    event->type == SRL_EVENT_SELECTION_CHANGED;
+	    event->type == SRL_EVENT_SELECTION_CHANGED	||
+	    event->type == SRL_EVENT_VISIBLE_DATA_CHANGED;
 }
 
 static gboolean
@@ -725,8 +738,7 @@ srl_is_tooltip_event (SRLEvent *event)
 {
     srl_assert (event);
     
-    return  event->type == SRL_EVENT_TOOLTIP_SHOW	||
-	    event->type == SRL_EVENT_TOOLTIP_HIDE;
+    return  event->type == SRL_EVENT_TOOLTIP_SHOW;
 }
 
 static gboolean
@@ -842,6 +854,40 @@ srl_notify_clients_obj (SRLEvent *event,
 				    (gchar*)srle_get_reason (event), &obj))
 	obj = NULL;
 
+    if (obj)
+    {
+	SRObjectRoles role;
+	sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+	
+	if ((role == SR_ROLE_TABLE_LINE || role == SR_ROLE_TABLE_COLUMNS_HEADER) &&
+	    srle_has_type (event, SRL_EVENT_FOCUS))
+	{
+	    SRObject *pobj = NULL;
+	    Accessible *pacc;
+ 
+	    pacc = Accessible_getParent ((Accessible*)srle_get_acc (event));
+
+	    if (pacc != srl_last_table)
+	    {
+		if (pacc)
+		    if (!sro_get_from_acc (pacc, SRL_EVENT_NAME_FOCUS, &pobj))
+			pobj = NULL;
+		if (pobj)
+		{
+		    SREvent *pev = NULL;
+		    pev = sre_new ();
+		    pev->type = type;
+		    pev->data = pobj;
+    		    pev->data_destructor = (SREventDataDestructor) sro_release_reference;
+		    srl_notify_all_clients (pev, 0);
+		    sre_release_reference (pev);
+		}
+	    }
+	    if (pacc)
+		Accessible_unref (pacc);
+	}
+    }
+
     if (obj)
     {
 	static SRObjectRoles last_role = SR_ROLE_UNKNOWN; /* to report all headers
@@ -1076,6 +1122,14 @@ srl_acc_has_toolkit (const Accessible *acc,
     return rv;    
 }
 
+static gboolean
+srle_is_for_watched_acc (SRLEvent *event)
+{
+    srl_assert (event);
+    
+    return srl_watched_acc == event->acc_ev->source;
+}
+
 
 static gboolean
 srle_is_for_focused_acc (SRLEvent *event)
@@ -1136,9 +1190,18 @@ srle_change_type (SRLEvent *event)
     }
     else if (srle_has_type (event, SRL_EVENT_TEXT_CARET_MOVED))
     {
-	if (!srle_is_for_focused_acc (event) &&
+	static gint last_link_index = -1;
+	gint new_link_index = sr_acc_get_link_index ((Accessible*)srle_get_acc (event));
+/*	if (!srle_is_for_focused_acc (event) &&
 	    srl_acc_has_toolkit (srle_get_acc (event), SRL_TOOLKIT_MOZILLA))
 	    event->type = SRL_EVENT_FOCUS;
+*/
+	if (!srle_is_for_focused_acc (event) &&
+	    !srle_is_for_watched_acc (event))
+	    event->type = SRL_EVENT_FOCUS2;
+	if (last_link_index != new_link_index)
+	    event->type = SRL_EVENT_FOCUS;
+	last_link_index = new_link_index;
     }
     else if (srle_has_type (event, SRL_EVENT_NAME_CHANGED))
     {
@@ -1381,13 +1444,6 @@ srl_selection_event_is_reported (SRLEvent *event)
     return rv;
 }
 
-static gboolean
-srle_is_for_watched_acc (SRLEvent *event)
-{
-    srl_assert (event);
-    
-    return srl_watched_acc == event->acc_ev->source;
-}
 
 
 static gboolean
@@ -1405,9 +1461,12 @@ srl_event_is_reported (SRLEvent *event)
 	srl_is_tooltip_event (event))
 	rv = TRUE;
     else if (srle_has_type (event, SRL_EVENT_FOCUS))
-	rv = srle_acc_has_state (event, SPI_STATE_FOCUSED);
+	rv = srle_acc_has_state (event, SPI_STATE_FOCUSED) ||
+	     srle_acc_has_state (event, SPI_STATE_TRANSIENT);
     else if (srle_has_type (event, SRL_EVENT_FOCUS1))
 	rv = TRUE;
+    else if (srle_has_type (event, SRL_EVENT_FOCUS2))
+	rv = TRUE;
     else if (srle_is_for_focused_acc (event) ||
 	     srle_is_for_watched_acc (event))
     {
@@ -1435,6 +1494,8 @@ srl_event_is_reported (SRLEvent *event)
 	else if (srle_has_type (event, SRL_EVENT_VALUE_CHANGED))
 	    rv = srle_acc_has_role (event, SPI_ROLE_SPIN_BUTTON) ||
 		    srle_acc_has_role (event, SPI_ROLE_SLIDER);
+	else if (srle_has_type (event, SRL_EVENT_VISIBLE_DATA_CHANGED))
+	    rv = TRUE;
     }
 
     if (srle_has_type (event, SRL_EVENT_WINDOW_DEACTIVATE))
@@ -1444,7 +1505,15 @@ srl_event_is_reported (SRLEvent *event)
 	      srle_is_for_watched_acc (event)))
     {
 	srl_assert (last_info);
-	srl_set_info (event->acc_ev->source, last_info);
+	if (srle_has_type (event, SRL_EVENT_TEXT_CHANGED_DELETE))
+	{
+	    last_info->text_caret_offset  = event->acc_ev->detail1;
+	    last_info->text_selections_count  = 0;
+	    last_info->text_char_count -= event->acc_ev->detail2;
+	    last_info->text_crt_selection_length = 0;
+	}
+	else
+	    srl_set_info (event->acc_ev->source, last_info);
 
 	if (srle_is_for_focused_acc (event))
 	    if (last_info != &last_info_focus)
@@ -1482,8 +1551,9 @@ srl_process_event (SRLEvent *event)
     srle_set_acc (event);
     srle_change_type (event);
 
-    if (srle_has_type (event, SRL_EVENT_FOCUS))
-	srl_set_last_focus ((Accessible*)srle_get_acc (event));
+    if (srle_has_type (event, SRL_EVENT_FOCUS) ||
+	srle_has_type (event, SRL_EVENT_FOCUS2))
+	    srl_set_last_focus ((Accessible*)srle_get_acc (event));
     else if (srle_has_type (event, SRL_EVENT_FOCUS1))
 	srl_set_last_focus (NULL);
 
