@@ -408,6 +408,9 @@ gboolean sro_get_state 		(const SRObject *obj, SRState *state, SRLong index);
 gboolean sro_get_relation	(const SRObject *obj, SRRelation *relation, SRLong index);
 gboolean sro_get_role_name	(const SRObject *obj, gchar **role_name,SRLong index);
 gboolean sro_get_name 		(const SRObject *obj, gchar **name, SRLong index);
+gboolean sro_get_column_header 	(const SRObject *obj, gchar **header_name, SRLong index);
+gboolean sro_get_row_header     (const SRObject *obj, gchar **header_name, SRLong index);
+gboolean sro_get_cell           (const SRObject *obj, gchar **cell, SRLong index);
 gboolean sro_get_description 	(const SRObject *obj, gchar **description, SRLong index);
 gboolean sro_get_parent 	(const SRObject *obj, SRObject **parent);
 gboolean sro_get_index_in_parent(const SRObject *obj, guint32 *index);
